<?php
/*
Plugin Name: Superadmin helper
Description: Set of utilities for managing multisite Wordpress installations. Logging, simple permban, etc.
Version: 1.3.8
Author: Zaantar
Author URI: http://zaantar.eu
Donate Link: http://zaantar.eu/financni-prispevek
Plugin URI: http://wordpress.org/extend/plugins/superadmin-helper
License: GPL2
*/

/*
    Copyright 2010-2011 Zaantar (email: zaantar@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


require_once plugin_dir_path( __FILE__ ).'includes/options.php';
require_once plugin_dir_path( __FILE__ ).'includes/logging.php';
require_once plugin_dir_path( __FILE__ ).'includes/permban.php';
require_once plugin_dir_path( __FILE__ ).'includes/permban-table.php';


/*****************************************************************************\
		I18N
\*****************************************************************************/


define( 'SUH_TEXTDOMAIN', 'superadmin-helper' );
define( "SUH_TXD", SUH_TEXTDOMAIN );

add_action( 'init', 'suh_load_textdomain' );

function suh_load_textdomain() {
	$plugin_dir = basename(dirname(__FILE__));
	load_plugin_textdomain( SUH_TEXTDOMAIN, false, $plugin_dir.'/languages' );
}


/* ************************************************************************* *\
		LOGGING INTERFACE
\* ************************************************************************* */

define( 'SUH_MAIL_LOGNAME', 'suh-mail' );

function suh_nag( $message ) {
	echo( '<div id="message" class="updated"><p>'.$message.'</p></div>' );
}

function suh_nagerr( $message ) {
	echo( '<div id="message" class="error"><p>'.$message.'</p></div>' );
}

function suh_log( $message, $category = 1 ) {
	if( !defined( 'WLS' ) or !wls_is_registered( 'suh-log' ) ) {
		$filename = dirname(__FILE__).'/log.txt';
		$file = fopen( $filename, 'ä́' );
		fwrite( $file, $message );
		fclose( $file );
	} else {
		wls_simple_log( 'suh-log', $message, $category );
	}
}



/* ************************************************************************* *\
    	SUPERADMIN CAN CHANGE USER'S PRIMARY BLOG
\* ************************************************************************* */


add_action( 'show_user_profile','suh_pb_show_user_profile' );
add_action( 'edit_user_profile','suh_pb_show_user_profile' );
add_action( 'personal_options_update', 'suh_pb_edit_user_profile' );
add_action( 'edit_user_profile_update', 'suh_pb_edit_user_profile' );

function suh_pb_show_user_profile( $user ) {
	if( !is_super_admin() || !is_multisite() ) {
		return;
	}
	?>
	<h3><?php _e( 'Superadmin mode', SUH_TEXTDOMAIN ); ?><!--Superadmin mód--></h3>
	<p><?php _e( 'Be very careful. Wrong value can do some damage.', SUH_TEXTDOMAIN ); ?><!--
		Při změnách těchto polí buďte velmi opatrní! Můžete tím toho nejspíš hodně zkazit. -->
	</p>
	<table class="form-table">
		<tr>
		    <th><label for="primary_blog"><?php _e( 'Primary blog', SUH_TEXTDOMAIN ); ?><!--Primární blog--></label></th>
		    <td>
		        <input type="text" name="primary_blog" id="primary_blog" value="<?php echo( get_user_meta( $user->ID, 'primary_blog', true ) ); ?>" class="regular-text">
		    </td>
		</tr>
	</table>
	<?php
}	


function suh_pb_edit_user_profile( $user_id ) {
	if( !is_super_admin() ) {
		return;
	}
	$pb = $_POST['primary_blog'];
	update_user_meta( $user_id, 'primary_blog', $pb );
}


/*****************************************************************************\
		USER'S LAST LOGIN
\*****************************************************************************/

add_action( 'wp_login', 'suh_save_last_login' );

function suh_save_last_login( $username ) {
	$userdata = get_userdatabylogin( $username );
	update_user_meta( $userdata->ID, 'suh_last_login', date( 'Y-m-d H:i' ) );
}


add_filter( 'wpmu_users_columns', 'suh_add_last_login_column' );
add_filter( 'manage_users_columns', 'suh_add_last_login_column' );

function suh_add_last_login_column( $columns ) {
	if( is_super_admin() or ( is_network_admin() && is_multisite() ) or ( !is_network_admin() && !is_multisite() ) ) {
		$columns['suh_last_login'] = __( 'Last login', SUH_TEXTDOMAIN );
	}
	add_action( 'manage_users_custom_column',  'suh_add_last_login_column_value', 20, 3);
	return $columns;
}


function suh_add_last_login_column_value( $value, $column_name, $user_id ) {
	if( $column_name == 'suh_last_login' ) {
		$last_login = get_user_meta( $user_id, 'suh_last_login', true );
		if( $last_login == '' ) {
			return 'n/a';
		} else {
			return $last_login;
		}
	}
}


/*
TODO

add_filter( 'manage_users_sortable_columns', 'suh_make_last_login_column_sortable' );
function suh_make_last_login_column_sortable( $columns ) {
	if( is_super_admin() or ( !is_network_admin() && !is_multisite() ) ) {
		$columns['suh_last_login'] = 'suh_last_login';
	}
	return $columns;
}


add_filter( 'request', 'suh_last_login_orderby' );
function suh_last_login_orderby( $vars ) {
	if( ( is_super_admin() or ( !is_network_admin() && !is_multisite() ) ) 
		&& isset( $vars['orderby'] ) && $vars['orderby'] == 'suh_last_login' ) {
		$vars = array_merge( $vars, array(
			'meta_key' => 'suh_last_login',
			'orderby' => 'meta_value'
		) );
	}
	return $vars;
}*/


/*****************************************************************************\
		MISCELANNEOUS
\*****************************************************************************/


function suh_get_ip() {
	if(!empty($_SERVER['HTTP_CLIENT_IP'])) {
		$ip_address = $_SERVER['HTTP_CLIENT_IP'];
	} else if(!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
		$ip_address = $_SERVER['HTTP_X_FORWARDED_FOR'];
	} else if(!empty($_SERVER['REMOTE_ADDR'])) {
		$ip_address = $_SERVER['REMOTE_ADDR'];
	} else {
		$ip_address = '';
	}
	if(strpos($ip_address, ',') !== false) {
		$ip_address = explode(',', $ip_address);
		$ip_address = $ip_address[0];
	}
	return esc_attr($ip_address);
}


/*****************************************************************************\
		VERSION COMPATIBILITY
\*****************************************************************************/


define( 'SUH_DB_VERSION', 4 );

add_action( 'plugins_loaded', 'suh_db_version_check' );

function suh_db_version_check() {
	$settings = suh_get_settings();
	extract( $settings );
	if( $db_version == SUH_DB_VERSION )
		return;
	$lock_name = 'SUH_DB_VERSION_update';
	if( $db_version < 2 ) {
		// convert from multiple site options to one
		$settings = array(
			'banned_message' => get_site_option( 'suh_banned_message' ),
			'is_permban_active' => ( get_site_option( 'suh_permban_active', 'false' ) == 'true' ),
			'permban_ready' => get_site_option( 'suh_permban_ready', false ),
			'db_version' => 2
		);
		suh_update_settings( $settings );
		delete_site_option( 'suh_banned_message' );
		delete_site_option( 'suh_permban_active' );
		delete_site_option( 'suh_permban_ready' );
		suh_log( 'SUH_DB_VERSION upgraded from 1 to 2.', 2 );
		$db_version = 2;
	}
	// after this db_version is at least 2
	if( $db_version < 3 ) {
		suh_log( 'Upgrading SUH_DB_VERSION from 2 to 3.', 1 );
		if( $permban_ready ) {
			// remove duplicate ip entries and alter permban table so that it is unique
			// (only if we have the table already created)	
			suh_log( 'Removing duplicate ip entries in suh_permban_table.', 1 );
			global $wpdb; 
			if( $wpdb->get_var( 'SELECT IS_FREE_LOCK(\''.$lock_name.'\')' ) != '1' ) {
				return;
			}
			$locked = $wpdb->get_var( 'SELECT GET_LOCK(\''.$lock_name.'\', 0)' );
			if( $locked != '1' ) {
				suh_log( 'Cannot get lock because: '.$wpdb->last_error, 1 );
				return;
			}
			if( $wpdb->query( 'ALTER IGNORE TABLE '.suh_permban_table().' ADD UNIQUE INDEX(ip)' ) === FALSE ) {
				suh_log( sprintf ( 'MySQL while upgrading SUH_DB_VERSION.<br/>last query: "%s"<br/>result: "%s"<br/>error: %s.',
					$wpdb->last_query, 
					print_r( $wpdb->last_result, true ), 
					$wpdb->last_error 
				), 5 );
				/*if( is_admin() ) {
					suh_nagerr( 'Database error when upgrading Superadmin Helper. Please contact plugin developer.' );
				}*/
				$wpdb->query( 'SELECT RELEASE_LOCK(\''.$lock_name.'\')' );
				return;
			}
			$wpdb->query( 'SELECT RELEASE_LOCK(\''.$lock_name.'\')' );

		}
		$settings['db_version'] = 3;
		suh_update_settings( $settings );
		suh_log( 'SUH_DB_VERSION upgraded from 2 to 3.', 2 );
	}
	if( $db_version < 4 ) {
		suh_log( 'Upgrading SUH_DB_VERSION from 3 to 4.', 1 );
		if( $permban_ready ) {
			global $wpdb;
			if( $wpdb->get_var( 'SELECT IS_FREE_LOCK(\''.$lock_name.'\')' ) != '1' ) {
				return;
			}
			$locked = $wpdb->get_var( 'SELECT GET_LOCK(\''.$lock_name.'\', 0)' );
			if( $locked != '1' ) {
				suh_log( 'Cannot get lock because: '.$wpdb->last_error, 1 );
				return;
			}
			if( $wpdb->query( "ALTER TABLE ".suh_permban_table()." ADD COLUMN (attempt_count INT DEFAULT 0, last_attempt DATETIME)" ) === FALSE ) {
				suh_log( sprintf ( 'MySQL while upgrading SUH_DB_VERSION.<br/>last query: "%s"<br/>result: "%s"<br/>error: %s.',
					$wpdb->last_query, 
					print_r( $wpdb->last_result, true ), 
					$wpdb->last_error 
				), 5 );
				/*if( is_admin() ) {
					suh_nagerr( 'Database error when upgrading Superadmin Helper. Please contact plugin developer.' );
				}*/
				$wpdb->query( 'SELECT RELEASE_LOCK(\''.$lock_name.'\')' );
				return;
			}
			$wpdb->query( 'SELECT RELEASE_LOCK(\''.$lock_name.'\')' );
		} else {
			suh_log( 'No action needed.', 1 );
		}
		$settings['db_version'] = 4;
		suh_update_settings( $settings );
		suh_log( 'SUH_DB_VERSION upgraded from 3 to 4.', 2 );
	}	
}



?>
